// GeekOS C code entry point
// Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
// $Revision: 1.4 $

// This is free software.  You are permitted to use,
// redistribute, and modify it as specified in the file "COPYING".

#include "bootinfo.h"
#include "string.h"
#include "screen.h"
#include "mem.h"
#include "tss.h"
#include "int.h"
#include "kthread.h"
#include "trap.h"
#include "timer.h"
#include "keyboard.h"
#include "user.h"

static void Init_BSS( void );
#if 0
static void Test_Join( unsigned long arg );
#endif

void Main( struct Boot_Info* bootInfo )
{
    Init_BSS();
    Init_Screen();
    Init_Mem( bootInfo );
    Init_TSS();
    Init_Interrupts();
    Init_Scheduler();
    Init_Traps();
    Init_Timer();
    Init_Keyboard();

    Set_Current_Attr( ATTRIB(BLACK, CYAN|BRIGHT) );
    Print( "Welcome to KMOS!\n" );

#if 0
    // Test Join() function
    {
	struct Kernel_Thread* child;
	Print( "Starting child...\n" );
	child = Start_Kernel_Thread( &Test_Join, 0, PRIORITY_NORMAL, FALSE );
	Join( child );
	Print( "Child has exited\n" );
    }
#endif

    // Start KMSH
    {
	if ( !Start_User_Program( g_userProgramTable[0], TRUE ) )
	    Print( "Couldn't start program a.c\n" );
    }

    // Now this thread is done.
    Exit();
}

// Initialize the .bss section of the executable image.
static void Init_BSS( void )
{
    char *bssStart, *bssEnd;

    // The windows versions of gcc use slightly different
    // names for the bss begin and end symbols than the Linux version.

#if defined(GNU_WIN32)
    extern char _bss_start__;
    extern char _bss_end__;
    bssStart = &_bss_start__;
    bssEnd = &_bss_end__;
#else
    extern char __bss_start;
    extern char end;
    bssStart = &__bss_start;
    bssEnd = &end;
#endif

    // Fill .bss with zeroes
    memset( bssStart, '\0', bssEnd - bssStart );
}

#if 0
static void Test_Join( unsigned long arg )
{
    Print( "This is only a test...\n" );
}
#endif
